
   import java.awt.*;
   import java.awt.event.*;

    abstract
    public class ProjectileWorld extends Frame
   {
      public static final int FrameWidth = 600;
      public static final int FrameHeight = 400;
   
      private int        angle;
      private String     message;
      private Projectile projectile;
      private Scrollbar  slider;
   
       public ProjectileWorld()
      {
         setSize ( FrameWidth, FrameHeight );
         setTitle( "Projectile World" );
      
         angle      = 45;
         message    = "Angle: " + angle;
         projectile = null;
      
         slider = new Scrollbar( Scrollbar.VERTICAL, angle, 5, 0, 90 );
         slider.addAdjustmentListener( new ScrollBarListener() );
         add( "East", slider );
      
         Button fire = new Button( "fire" );
         fire.addActionListener( new FireButtonListener() );
         add( "North", fire );
      }
   
       public static int dy( int y )
      {
         return FrameHeight - y;
      }
   
       public void paint( Graphics g )
      {
         drawCannon    ( g );
         drawTarget    ( g );
         drawProjectile( g );
         writeMessage  ( g );
      }
   
   /* -------------- helper methods -------------- */
   
       protected void drawCannon( Graphics g )
      {
         int x = 20;
         int y = 15;
      
         double radianAngle = angle * Math.PI / 180.0;
      
         int lv = (int) (30 * Math.sin(radianAngle));
         int lh = (int) (30 * Math.cos(radianAngle));
         int sv = (int) (10 * Math.sin(radianAngle + Math.PI/2));
         int sh = (int) (10 * Math.cos(radianAngle + Math.PI/2));
      
         g.setColor(Color.green);
         g.drawLine(x, dy(y), x+lh, dy(y+lv));
         g.drawLine(x+lh, dy(y+lv), x+lh+sh, dy(y+lv+sv));
         g.drawLine(x+lh+sh, dy(y+lv+sv), x+sh, dy(y+sv));
         g.drawLine(x+sh, dy(y+sv), x, dy(y));
         g.drawOval(x-8, dy(y+10), 12, 12);
      }
   
       protected void drawTarget( Graphics g )
      {
         g.setColor(Color.red);
         g.fillRoundRect( FrameWidth-100, dy(12), 50, 10, 6, 6 );
      }
   
       protected void drawProjectile( Graphics g )
      {
         if ( projectile != null )
         { 
            projectile.move();
            projectile.paint( g );
         
            if ( projectile.northOf( new Point ( 0, FrameHeight ) ) ) {
               repaint();
               try
               {
                  Thread.sleep( 30 );
               }
                   catch (InterruptedException e)
                  {
                     System.exit( 0 );
                  }
            } 
            else
            {
               int targetX = FrameWidth - 100;
               if ( projectile.eastOf( new Point(targetX   ,0) )  &&
                    projectile.westOf( new Point(targetX+50,0) ) )
                  message = "You Hit It!";
               else
                  message = "Missed!";
               projectile = null;
            }
         }
      }
   
       protected void writeMessage( Graphics g )
      {
         g.drawString( message, FrameWidth/2, FrameHeight/2 );
      }
   
   /* -------------- inner classes -------------- */
   
       private class FireButtonListener implements ActionListener
      {
          public void actionPerformed( ActionEvent e )
         {
            projectile = makeProjectile( angle );
            repaint();
         }
      }
   
       private class ScrollBarListener implements AdjustmentListener
      {
          public void adjustmentValueChanged (AdjustmentEvent e)
         {
            angle   = slider.getValue();
            message = "Angle: " + angle;
            repaint();
         }
      }
   
   /* -------------- abstract methods -------------- */
   
       abstract
       protected Projectile makeProjectile( int startAngle );
   }
